@@ -13,7 +13,6 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
 
-import org.gnu.gtk.event.GtkEventType;
 
 /**
  * An EventMap is a Java-GNOME implementation class which encapsulates all
@@ -35,10 +34,10 @@ public class EventMap {
 	private class Entry {
 		String signal;
 		String method;
-		GtkEventType eventType;
+		EventType eventType;
 		Class listenerClass;
 		HashMap handlerIDs;
-		Entry(String s, String m, GtkEventType t, Class l) {
+		Entry(String s, String m, EventType t, Class l) {
 			signal = s;
 			method = m;
 			eventType = t;
@@ -55,7 +54,7 @@ public class EventMap {
         }
 	}
 
-	public void addEvent(String signal, String method, GtkEventType eventType, Class listenerClass) {
+	public void addEvent(String signal, String method, EventType eventType, Class listenerClass) {
 		// do not add events with null event types
 		if (null == eventType)  {
 			System.out.println("Signal " + signal + " has a null event type.  Please fix!");
@@ -80,7 +79,7 @@ public class EventMap {
 	/**
 	 * Connect an event handler of the type provided.
 	 */
-	public void initialize(GObject source, GtkEventType type) {
+	public void initialize(GObject source, EventType type) {
 		Entry e = (Entry)events.get(type);
 		if (null == e)
 			return;
@@ -90,7 +89,7 @@ public class EventMap {
 	/**
 	 * Disconnect an event handler of the type provided.
 	 */
-	public void uninitialize(GObject source, GtkEventType type) {
+	public void uninitialize(GObject source, EventType type) {
 		Entry e = (Entry)events.get(type);
 		if (null == e)
 			return;
@@ -115,8 +114,8 @@ public class EventMap {
 	/**
 	 * Get the event type for a given signal.
 	 */
-	public GtkEventType getEventType(String signal) {
-		GtkEventType rv = null;
+	public EventType getEventType(String signal) {
+		EventType rv = null;
 		Iterator i = events.entrySet().iterator();
 		while (i.hasNext()) {
 			Map.Entry me = (Map.Entry)i.next();
