@@ -287,7 +287,7 @@ public class Notebook extends Container {
 	    Handle hndl = Notebook.gtk_notebook_get_nth_page(getHandle(), pageNum);
 		if (null == hndl)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Widget)obj;
 		return new Widget(hndl);
@@ -423,7 +423,7 @@ public class Notebook extends Container {
 		return true;
 	}
 
-	private void handleSwitchPage(int page, int pageNum) {
+	private void handleSwitchPage(Handle page, int pageNum) {
 		fireNotebookEvent(new NotebookEvent(this, NotebookEvent.Type.SWITCH_PAGE, pageNum));
 	}
 
@@ -481,19 +481,15 @@ public class Notebook extends Container {
 	native static final protected void gtk_notebook_set_menu_label(Handle notebook, Handle child, Handle menuLabel);
 	native static final protected void gtk_notebook_set_menu_label_text(Handle notebook, Handle child, String menuText);
 	native static final protected String gtk_notebook_get_menu_label_text(Handle notebook, Handle child);
-	native static final protected void gtk_notebook_query_tab_label_packing(
-	        Handle notebook,
-	        Handle child,
-	        boolean[] expand,
-	        boolean[] fill,
-	        int[] packType);
-	native static final protected void gtk_notebook_set_tab_label_packing(
-	        Handle notebook,
-	        Handle child,
-	        boolean expand,
-	        boolean fill,
-	        int packType);
+	native static final protected void gtk_notebook_query_tab_label_packing(Handle notebook,Handle child,boolean[] expand,boolean[] fill,int[] packType);
+	native static final protected void gtk_notebook_set_tab_label_packing(Handle notebook,Handle child,boolean expand,boolean fill,int packType);
 	native static final protected void gtk_notebook_reorder_child(Handle notebook, Handle child, int position);
 	native static final protected int gtk_notebook_get_n_pages(Handle notebook);
 
+    /* Deprecated functions.
+    native static final private void gtk_notebook_set_homogeneous_tabs(Handle notebook, boolean homogeneous);
+    native static final private void gtk_notebook_set_tab_border(Handle notebook, int border_width);
+    native static final private void gtk_notebook_set_tab_hborder(Handle notebook, int tab_hborder);
+    native static final private void gtk_notebook_set_tab_vborder(Handle notebook, int tab_vborder);
+    */
 }
