@@ -130,7 +130,7 @@ public class TextView extends Container {
 	 */
 	public TextBuffer getBuffer() {
 	    Handle hndl = gtk_text_view_get_buffer(getHandle());
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (TextBuffer)obj;
 		return new TextBuffer(hndl);
@@ -380,6 +380,14 @@ public class TextView extends Container {
 	public boolean getOverwrite(){
 		return gtk_text_view_get_overwrite (getHandle());
 	}
+    
+    public Adjustment getHAdjustment() {
+        return new Adjustment(getHAdjustment(getHandle()));
+    }
+    
+    public Adjustment getVAdjustment() {
+        return new Adjustment(getVAdjustment(getHandle())); 
+    }
 
 	public Class getEventListenerClass(String signal) {
 		Class cls = evtMap.getEventListenerClass(signal);
@@ -417,7 +425,7 @@ public class TextView extends Container {
 		anEvtMap.addEvent("toggle_overwrite", "handleToggleOverwrite", TextViewEvent.Type.TOGGLE_OVERWRITE, TextViewListener.class);
 	}
 
-	/****************************************
+	/* ***************************************
 	 * TextView event handling.
 	 ****************************************/
 	/**
@@ -527,7 +535,7 @@ public class TextView extends Container {
 
 	private void handlePopulatePopup(Handle menu) {
 		TextViewEvent tve = new TextViewEvent(this, TextViewEvent.Type.POPULATE_POPUP);
-		GObject obj = retrieveGObject(menu);
+		GObject obj = getGObjectFromHandle(menu);
 		if (null != obj)
 			tve.setMenu((Menu)obj);
 		else
@@ -535,7 +543,7 @@ public class TextView extends Container {
 		fireTextViewEvent(tve);
 	}
 
-	private void handleSetScrollAdjustments(int hajd, int vadj) {
+	private void handleSetScrollAdjustments(Handle hajd, Handle vadj) {
 		fireTextViewEvent(new TextViewEvent(this, TextViewEvent.Type.SET_SCROLL_ADJUSTMENTS));
 	}
 
@@ -554,6 +562,8 @@ public class TextView extends Container {
 		return new Type(gtk_text_view_get_type());
 	}
 
+    native static final protected Handle getHAdjustment(Handle textView);
+    native static final protected Handle getVAdjustment(Handle textView);
 	native static final protected int gtk_text_view_get_type();
 	native static final protected Handle gtk_text_view_new();
 	native static final protected Handle gtk_text_view_new_with_buffer(Handle buffer);
