@@ -96,7 +96,7 @@ import org.gnu.gtk.event.TreeViewListener;
  * </dl>
  * <p>The CellRenderer's need data to be able to display. This is set using the
  * {@link TreeViewColumn#addAttributeMapping(CellRenderer,
- * CellRendererAttribute, int)}. The renderer attributes vary with each
+ * CellRendererAttribute, DataColumn)}. The renderer attributes vary with each
  * renderer, for example CellRendererText has a TEXT attribute for the text the
  * be displayed. The final parameter is for the dataBlock in the store in which
  * the data is contained.
@@ -109,6 +109,8 @@ import org.gnu.gtk.event.TreeViewListener;
  */
 public class TreeView extends Container {
 
+    private TreeViewRowSeparatorMethod separatorFunc = null;
+
 	/**
 	 * Create a new TreeView object.
 	 */
@@ -135,7 +137,7 @@ public class TreeView extends Container {
 	 */
 	public TreeModel getModel() {
 	    Handle hndl = gtk_tree_view_get_model(getHandle());
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (TreeModel)obj;
 		return new TreeModel(hndl);
@@ -176,7 +178,7 @@ public class TreeView extends Container {
 	 */
 	public TreeSelection getSelection() {
 	    Handle hndl = gtk_tree_view_get_selection(getHandle());
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (TreeSelection)obj;
 		return new TreeSelection(hndl);
@@ -188,7 +190,7 @@ public class TreeView extends Container {
 	 */
 	public Adjustment getHAdjustment() {
 	    Handle hndl = gtk_tree_view_get_hadjustment(getHandle());
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Adjustment)obj;
 		return new Adjustment(hndl);
@@ -263,7 +265,7 @@ public class TreeView extends Container {
 
 	/**
 	 * Gets the column at the given position in the tree view.
-	 * @param n  The position of the column, counting from 0.
+	 * @param position  The position of the column, counting from 0.
 	 * @return The TreeViewColumn, or <code>null</code> if the position is outside the range 
 	 * of columns.
 	 */
@@ -271,7 +273,7 @@ public class TreeView extends Container {
 	    Handle h = gtk_tree_view_get_column(getHandle(), position);
 		if (h == null)
 			return null;
-		GObject obj = retrieveGObject(h);
+		GObject obj = getGObjectFromHandle(h);
 		if (null != obj)
 			return (TreeViewColumn)obj;
 		return new TreeViewColumn(h);
@@ -302,7 +304,7 @@ public class TreeView extends Container {
 	    Handle hndl = gtk_tree_view_get_expander_column(getHandle());
 		if (hndl == null)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (TreeViewColumn)obj;
 		return new TreeViewColumn(hndl);
@@ -314,11 +316,12 @@ public class TreeView extends Container {
 	 */
 
 	/**
-	 * Moves the alignments of the view to the position specified by column and 
-	 * path. . rowAlign determines where the row is placed, 
-	 * and colAlign determines where column is placed. Both are expected to be 
-	 * between 0.0 and 1.0. 0.0 means left/top alignment, 1.0 means 
-	 * right/bottom alignment, 0.5 means center.
+	 * Moves the alignments of the view to the position specified
+	 * by </tt>column</tt> and <tt>path</tt>. <tt>rowAlign</tt>
+	 * determines where the row is placed, and <tt>colAlign</tt>
+	 * determines where column is placed. Both are expected to be
+	 * between 0.0 and 1.0. 0.0 means left/top alignment, 1.0
+	 * means right/bottom alignment, 0.5 means center.
 	 *
 	 * @param path The path of the row to move to
 	 * @param column The TreeViewColumn to move horizontally to
@@ -400,7 +403,7 @@ public class TreeView extends Container {
 	public TreeViewColumn getCursorColumn() {
 		Handle column = GObject.getNullHandle();
 		gtk_tree_view_get_cursor(getHandle(), null, column);
-		GObject obj = retrieveGObject(column);
+		GObject obj = getGObjectFromHandle(column);
 		if (null != obj)
 			return (TreeViewColumn)obj;
 		return new TreeViewColumn(column);
@@ -512,7 +515,7 @@ public class TreeView extends Container {
 			return null;
 		TreeViewColumn[] columns = new TreeViewColumn[hndls.length];
 		for (int i = 0; i < hndls.length; i++) {
-			GObject obj = retrieveGObject(hndls[i]);
+			GObject obj = getGObjectFromHandle(hndls[i]);
 			if (null != obj)
 				columns[i] = (TreeViewColumn)obj;
 			else
@@ -676,6 +679,41 @@ public class TreeView extends Container {
         return gtk_tree_view_get_hover_expand( getHandle() );
     }
 
+    /**
+     * Set the object which is used to determine whether a row should be drawn
+     * as a separator.
+     *
+     * @param method The <tt>TreeViewRowSeparatorMethod</tt> or
+     * <tt>null</tt> if no separators are to be drawn (the default).
+     */
+    public void setRowSeparatorMethod( TreeViewRowSeparatorMethod method ) {
+        separatorFunc = method;
+        if ( method == null ) {
+            gtk_tree_view_set_row_separator_func( getHandle(), null,
+                                                  "handleSeparatorFunc" );
+        } else {
+            gtk_tree_view_set_row_separator_func( getHandle(), this,
+                                                  "handleSeparatorFunc" );
+        }
+    }
+
+    protected boolean handleSeparatorFunc( Handle model, Handle iter ) {
+        TreeModel mod = (TreeModel)getGObjectFromHandle(model);
+        TreeIter it = new TreeIter( iter, mod );
+        return separatorFunc.isSeparator( mod, it );
+    }
+
+    /**
+     * Get the object which is used to determine whether a row should be
+     * drawn as a separator.
+     *
+     * @return The <tt>TreeViewRowSeparatorMethod</tt> or <tt>null</tt>
+     * if no method has been set.
+     */
+    public TreeViewRowSeparatorMethod getRowSeparatorMethod() {
+        return separatorFunc;
+    }
+
     //
     // Event Handler Related code
     //
@@ -765,7 +803,7 @@ public class TreeView extends Container {
 	private void handleRowActivated(Handle path, Handle column) {
 		TreeViewEvent event = new TreeViewEvent(this, TreeViewEvent.Type.ROW_ACTIVATED);
 		event.setTreePath(new TreePath(path));
-		GObject obj = retrieveGObject(column);
+		GObject obj = getGObjectFromHandle(column);
 		if (null != obj)
 			event.setTreeColumn((TreeViewColumn)obj);
 		else
@@ -936,7 +974,7 @@ public class TreeView extends Container {
     native static final private boolean gtk_tree_view_get_hover_selection(Handle tree_view);
     native static final private void gtk_tree_view_set_hover_expand(Handle tree_view, boolean expand);
     native static final private boolean gtk_tree_view_get_hover_expand(Handle tree_view);
+    native static final private void gtk_tree_view_set_row_separator_func(Handle tree_view, TreeView tview, String callback );
     //native static final private int gtk_tree_view_get_row_separator_func(Handle tree_view);
-    //native static final private void gtk_tree_view_set_row_separator_func(Handle tree_view, int func, gpointer data, int destroy);
 
 }
