@@ -22,8 +22,8 @@ import org.gnu.gdk.EventButton;
 import org.gnu.gdk.EventCrossing;
 import org.gnu.gdk.EventExpose;
 import org.gnu.gdk.EventKey;
-import org.gnu.gdk.EventNoExpose;
 import org.gnu.gdk.EventMotion;
+import org.gnu.gdk.EventNoExpose;
 import org.gnu.gdk.EventScroll;
 import org.gnu.gdk.ModifierType;
 import org.gnu.gdk.Pixbuf;
@@ -33,7 +33,9 @@ import org.gnu.gdk.Rectangle;
 import org.gnu.gdk.Screen;
 import org.gnu.gdk.Window;
 import org.gnu.glib.EventMap;
+import org.gnu.glib.EventType;
 import org.gnu.glib.GObject;
+import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.DragDestinationListener;
 import org.gnu.gtk.event.DragSourceListener;
@@ -41,7 +43,6 @@ import org.gnu.gtk.event.ExposeEvent;
 import org.gnu.gtk.event.ExposeListener;
 import org.gnu.gtk.event.FocusEvent;
 import org.gnu.gtk.event.FocusListener;
-import org.gnu.gtk.event.GtkEventType;
 import org.gnu.gtk.event.KeyEvent;
 import org.gnu.gtk.event.KeyListener;
 import org.gnu.gtk.event.LifeCycleEvent;
@@ -55,11 +56,12 @@ import org.gnu.pango.FontDescription;
 
 /**
  * GtkWidget is the base class of all widgets.
- * 
  */
-
 public class Widget extends GtkObject {
 
+    // Cached class array used in makeWidget.
+    private static Class[] handleClass = new Class[] { Handle.class };
+
 	/**
 	 * The list of objects interested in focus events.
 	 */
@@ -106,7 +108,7 @@ public class Widget extends GtkObject {
 	 * 
 	 * @param handle The handle that represents a pointer to a native resource.
 	 */
-	public Widget(int handle) {
+	public Widget(Handle handle) {
 		super(handle);
 	}
 
@@ -209,10 +211,10 @@ public class Widget extends GtkObject {
 	 * @return The parent container for this widget.
 	 */
 	public Widget getParent() {
-		int hndl = gtk_widget_get_parent(getHandle());
-		if (0 == hndl)
+	    Handle hndl = gtk_widget_get_parent(getHandle());
+		if (null == hndl)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Widget)obj;
 		return new Widget(hndl);
@@ -224,8 +226,8 @@ public class Widget extends GtkObject {
 	 * @return Returns the widget's parent window.
 	 */
 	public Window getParentWindow() {
-		int hndl = gtk_widget_get_parent_window(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_widget_get_parent_window(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Window)obj;
 		return new Window(hndl);
@@ -244,10 +246,10 @@ public class Widget extends GtkObject {
 	 * gdk window.
 	 */
 	public Window getWindow() {
-		int hndl = Widget.gtk_widget_get_window(getHandle());
-		if(hndl == 0)
+	    Handle hndl = Widget.gtk_widget_get_window(getHandle());
+		if(hndl == null)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		return obj != null ? (Window)obj : new Window(hndl);
 	}
 
@@ -258,11 +260,11 @@ public class Widget extends GtkObject {
 	 * @return The intersection.  If the areas don't intersect this method returns null.
 	 */
 	public Rectangle intersect(Rectangle area) {
-		int[] rect = new int[1];
+	    Handle rect = GObject.getNullHandle();
 		boolean val = Widget.gtk_widget_intersect(getHandle(), area.getHandle(), rect);
 		if (false == val)
 			return null;
-		return new Rectangle(rect[0]);
+		return new Rectangle(rect);
 	}
 
 	/**
@@ -337,10 +339,10 @@ public class Widget extends GtkObject {
 	 * @return The topmost widget.
 	 */
 	public Widget getToplevel() {
-		int hndl = gtk_widget_get_toplevel(getHandle());
-		if (0 == hndl)
+	    Handle hndl = gtk_widget_get_toplevel(getHandle());
+		if (null == hndl)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Widget)obj;
 		return new Widget(hndl);
@@ -352,8 +354,8 @@ public class Widget extends GtkObject {
 	 * @return The colormap used for this widget.
 	 */
 	public Colormap getColormap() {
-		int colorHandle = Widget.gtk_widget_get_colormap(getHandle());
-		GObject obj = retrieveGObject(colorHandle);
+	    Handle colorHandle = Widget.gtk_widget_get_colormap(getHandle());
+		GObject obj = getGObjectFromHandle(colorHandle);
 		if (null != obj)
 			return (Colormap)obj;
 		return new Colormap(colorHandle);
@@ -494,7 +496,7 @@ public class Widget extends GtkObject {
 	 * @return The current RcStyle for the Widget.
 	 */
 	public RcStyle getModifierStyle() {
-		int hndl = Widget.gtk_widget_get_modifier_style(getHandle());
+	    Handle hndl = Widget.gtk_widget_get_modifier_style(getHandle());
 		return new RcStyle(hndl);
 	}
 
@@ -504,8 +506,8 @@ public class Widget extends GtkObject {
 	 * @return The current Style for the Widget.
 	 */
 	public Style getStyle() {
-		int hndl = Widget.gtk_widget_get_style(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = Widget.gtk_widget_get_style(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Style)obj;
 		return new Style(hndl);
@@ -518,7 +520,7 @@ public class Widget extends GtkObject {
 	 * @return A context that can be used for drawig text.
 	 */
 	public Context createContext() {
-		int pc = Widget.gtk_widget_create_pango_context(getHandle());
+	    Handle pc = Widget.gtk_widget_create_pango_context(getHandle());
 		return new Context(pc);
 	}
 
@@ -531,8 +533,8 @@ public class Widget extends GtkObject {
 	 * @return A context that can be used for drawing text.
 	 */
 	public Context getContext() {
-		int pc = Widget.gtk_widget_get_pango_context(getHandle());
-		GObject obj = retrieveGObject(pc);
+	    Handle pc = Widget.gtk_widget_get_pango_context(getHandle());
+		GObject obj = getGObjectFromHandle(pc);
 		if (null != obj)
 			return (Context)obj;
 		return new Context(pc);
@@ -546,7 +548,7 @@ public class Widget extends GtkObject {
 	 * @return The layout that can be used for drawing text.
 	 */
 	public org.gnu.pango.Layout createLayout(String text) {
-		int layout = Widget.gtk_widget_create_pango_layout(getHandle(), text);
+	    Handle layout = Widget.gtk_widget_create_pango_layout(getHandle(), text);
 		return new org.gnu.pango.Layout(layout);
 	}
 
@@ -563,11 +565,11 @@ public class Widget extends GtkObject {
 	/**
 	 * Returns the accessible object associated with this widget.
 	 * 
-	 * @returns The AtkObject.
+	 * @return The AtkObject.
 	 */
 	public AtkObject getAccessible() {
-		int ao = Widget.gtk_widget_get_accessible(getHandle());
-		GObject obj = retrieveGObject(ao);
+	    Handle ao = Widget.gtk_widget_get_accessible(getHandle());
+		GObject obj = getGObjectFromHandle(ao);
 		if (null != obj)
 			return (AtkObject)obj;
 		return new AtkObject(ao);
@@ -578,8 +580,8 @@ public class Widget extends GtkObject {
 	}
 
 	public Screen getScreen() {
-		int hndl = gtk_widget_get_screen(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_widget_get_screen(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Screen)obj;
 		return new Screen(hndl);
@@ -590,16 +592,16 @@ public class Widget extends GtkObject {
 	}
 
 	public Display getDisplay() {
-		int hndl = gtk_widget_get_display(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_widget_get_display(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Display)obj;
 		return new Display(hndl);
 	}
 
 	public Window getRootWindow() {
-		int hndl = gtk_widget_get_root_window(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_widget_get_root_window(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Window)obj;
 		return new Window(hndl);
@@ -636,8 +638,8 @@ public class Widget extends GtkObject {
 		return rv;
 	}
 
-	public GtkEventType getEventType(String signal) {
-		GtkEventType et = evtMap.getEventType(signal);
+	public EventType getEventType(String signal) {
+		EventType et = evtMap.getEventType(signal);
 		if (et == null) et = super.getEventType( signal );
 		return et;
 	}
@@ -766,11 +768,11 @@ public class Widget extends GtkObject {
 		fireLifeCycleEvent(new LifeCycleEvent(this, LifeCycleEvent.Type.UNREALIZE));
 	}
 
-	private boolean handleDestroy(int event) {
+	private boolean handleDestroy(Handle event) {
 		return fireBooleanLifeCycleEvent(new LifeCycleEvent(this, LifeCycleEvent.Type.DESTROY), true );
 	}
 
-	protected boolean handleDelete(int event) {
+	protected boolean handleDelete(Handle event) {
 		return fireBooleanLifeCycleEvent(new LifeCycleEvent(this, LifeCycleEvent.Type.DELETE), false);
 	}
 
@@ -836,11 +838,11 @@ public class Widget extends GtkObject {
 		return retval;
 	}
 
-	protected boolean handleFocusIn(int event) {
+	protected boolean handleFocusIn(Handle event) {
 		return fireFocusEvent(new FocusEvent(this, FocusEvent.Type.FOCUS_IN));
 	}
 
-	protected boolean handleFocusOut(int event) {
+	protected boolean handleFocusOut(Handle event) {
 		return fireFocusEvent(new FocusEvent(this, FocusEvent.Type.FOCUS_OUT));
 	}
 
@@ -898,11 +900,11 @@ public class Widget extends GtkObject {
 		return retval;
 	}
 
-	protected boolean handleKeyPressed(int event) {
+	protected boolean handleKeyPressed(Handle event) {
 		return fireKeyEvent(new KeyEvent(this, KeyEvent.Type.KEY_PRESSED, new EventKey(event)));
 	}
 
-	protected boolean handleKeyReleased(int event) {
+	protected boolean handleKeyReleased(Handle event) {
 		return fireKeyEvent(new KeyEvent(this, KeyEvent.Type.KEY_RELEASED, new EventKey(event)));
 	}
 
@@ -966,23 +968,23 @@ public class Widget extends GtkObject {
 		return retval;
 	}
 
-	protected boolean handleButtonPressed(int event) {
+	protected boolean handleButtonPressed(Handle event) {
 		return fireMouseEvent(new MouseEvent(this, MouseEvent.Type.BUTTON_PRESS, new EventButton(event)));
 	}
 
-	protected boolean handleButtonReleased(int event) {
+	protected boolean handleButtonReleased(Handle event) {
 		return fireMouseEvent(new MouseEvent(this, MouseEvent.Type.BUTTON_RELEASE, new EventButton(event)));
 	}
 
-	protected boolean handleScroll(int event) {
+	protected boolean handleScroll(Handle event) {
 		return fireMouseEvent(new MouseEvent(this, MouseEvent.Type.WHEEL_SCROLL, new EventScroll(event)));
 	}
 
-	protected boolean handleMouseEnter(int event) {
+	protected boolean handleMouseEnter(Handle event) {
 		return fireMouseEvent(new MouseEvent(this, MouseEvent.Type.ENTER, new EventCrossing(event)));
 	}
 
-	protected boolean handleMouseLeave(int event) {
+	protected boolean handleMouseLeave(Handle event) {
 		return fireMouseEvent(new MouseEvent(this, MouseEvent.Type.LEAVE, new EventCrossing(event)));
 	}
 
@@ -1015,28 +1017,37 @@ public class Widget extends GtkObject {
 	* @throws ClassNotFoundException if a Java-GNOME class cannot
 	*         be found for the specified handle.
 	*/
-	public static Widget makeWidget(int handle) throws ClassNotFoundException {
+	public static Widget makeWidget(Handle handle) throws ClassNotFoundException {
 		String widgetClass = getTypeClass(handle);
 		if (widgetClass == null)
 			throw new ClassNotFoundException("No GTK class name: invalid handle?");
 		try {
 			Class cls = Class.forName(widgetClass);
-			java.lang.reflect.Constructor constr = cls.getConstructor(new Class[] { Integer.TYPE });
-			Widget cons =  (Widget)constr.newInstance(new Object[] { new Integer(handle)});
+			java.lang.reflect.Constructor constr = 
+                            cls.getConstructor(handleClass);
+			Widget cons =  (Widget)constr.newInstance(new Object[] { handle});
 			return cons;
 		} catch (ClassNotFoundException e) {
 			throw e;
 		} catch (NoSuchMethodException e) {
-			throw new ClassNotFoundException("could not find constructor: " + widgetClass + "(int)");
+			throw new ClassNotFoundException("could not find constructor: " + widgetClass + "(Handle)");
 		} catch (Exception e) {
 			throw new ClassNotFoundException(e.toString());
 		}
 	}
 
-	private static String getTypeClass(int handle) {
+	private static String getTypeClass(Handle handle) {
 		String widgetType = Gtk.getTypeName(handle);
-		if (widgetType.startsWith("Gtk"))
-			return "org.gnu.gtk." + widgetType.substring(3);
+		if (widgetType.startsWith("Gtk")) {
+                    String wname = widgetType.substring(3);
+                    // Check for naming exceptions.
+                    if ( wname.equals( "Statusbar" ) ) {
+                        wname = "StatusBar";
+                    } else if ( wname.equals( "Toolbar" ) ) {
+                        wname = "ToolBar";
+                    }
+                    return "org.gnu.gtk." + wname;
+                }
 		else if (widgetType.startsWith("Gnome"))
 			return "org.gnu.gnome." + widgetType.substring(5);
 
@@ -1054,13 +1065,13 @@ public class Widget extends GtkObject {
 	 * @param targets An array of names of drag and drop operations accepted
 	 * (these are made up by the application designers. Widgets must set as drag
 	 * sources and destinations with one of these the same)
-	 * @param action Drag action - determines which events will be called.
+	 * @param actions Drag action - determines which events will be called.
 	 */
 	public void setDragSource(
 		ModifierType buttons,
 		TargetEntry[] targets,
 		DragAction actions) {
-		int[] targets_g = new int[targets.length];
+	    Handle[] targets_g = new Handle[targets.length];
 		for (int i = 0; i < targets.length; i++)
 			targets_g[i] = targets[i].getHandle();
 		gtk_drag_source_set(getHandle(), buttons.getValue(), targets_g, actions.getValue());
@@ -1091,7 +1102,7 @@ public class Widget extends GtkObject {
 	 * @param flags 
 	 */
 	public void setDragDestination(DestDefaults flags, TargetEntry[] targets, org.gnu.gdk.DragAction actions) {
-		int[] targets_g = new int[targets.length];
+	    Handle[] targets_g = new Handle[targets.length];
 		for (int i = 0; i < targets.length; i++)
 			targets_g[i] = targets[i].getHandle();
 		gtk_drag_dest_set(getHandle(), flags.getValue(), targets_g, actions.getValue());
@@ -1169,15 +1180,15 @@ public class Widget extends GtkObject {
 		}
 	}
 
-	protected void handleDragDestination(int context, int x, int y, int data, int id, int time) {
+	protected void handleDragDestination(Handle context, int x, int y, Handle data, int id, int time) {
 		SelectionData sd;
-		GObject obj = retrieveGObject(data);
+		GObject obj = getGObjectFromHandle(data);
 		if (null != obj)
 			sd = (SelectionData)obj;
 		else
 			sd = new SelectionData(data);
 		DragContext dc;
-		GObject obj2 = retrieveGObject(context);
+		GObject obj2 = getGObjectFromHandle(context);
 		if (null != obj2)
 			dc = (DragContext)obj2;
 		else
@@ -1232,15 +1243,15 @@ public class Widget extends GtkObject {
 		}
 	}
 
-	protected void handleDragDataGet(int context, int data, int id, int time) {
+	protected void handleDragDataGet(Handle context, Handle data, int id, int time) {
 		SelectionData sd;
-		GObject obj = retrieveGObject(data);
+		GObject obj = getGObjectFromHandle(data);
 		if (null != obj)
 			sd = (SelectionData)obj;
 		else
 			sd = new SelectionData(data);
 		DragContext dc;
-		GObject obj2 = retrieveGObject(context);
+		GObject obj2 = getGObjectFromHandle(context);
 		if (null != obj2)
 			dc = (DragContext)obj2;
 		else
@@ -1306,11 +1317,11 @@ public class Widget extends GtkObject {
 		return retval;
 	}
 
-	protected boolean handleExpose(int event) {
+	protected boolean handleExpose(Handle event) {
 		return fireExposeEvent(new ExposeEvent(this, new EventExpose(event)));
 	}
 
-	protected boolean handleNoExpose(int event) {
+	protected boolean handleNoExpose(Handle event) {
 		return fireExposeEvent(new ExposeEvent(this, new EventNoExpose(event)));
 	}
 
@@ -1370,140 +1381,134 @@ public class Widget extends GtkObject {
 		return retval;
 	}
 
-	protected boolean handleMouseMotion(int event) {
+	protected boolean handleMouseMotion(Handle event) {
 		return fireMouseMotionEvent(new MouseMotionEvent(this, new EventMotion(event)));
 	}
 
-	/****************************************
-	 * BEGINNING OF JNI CODE
-	 ****************************************/
 	native static final protected int gtk_widget_get_type();
-	native static final protected int gtk_widget_ref(int widget);
-	native static final protected void gtk_widget_unref(int widget);
-	native static final protected void gtk_widget_destroy(int widget);
-	native static final protected void gtk_widget_destroyed(int widget, int[] widgetPointer);
-	native static final protected void gtk_widget_unparent(int widget);
-	native static final protected void gtk_widget_show(int widget);
-	native static final protected void gtk_widget_show_now(int widget);
-	native static final protected void gtk_widget_hide(int widget);
-	native static final protected void gtk_widget_show_all(int widget);
-	native static final protected void gtk_widget_hide_all(int widget);
-	native static final protected void gtk_widget_map(int widget);
-	native static final protected void gtk_widget_unmap(int widget);
-	native static final protected void gtk_widget_realize(int widget);
-	native static final protected void gtk_widget_unrealize(int widget);
-	native static final protected void gtk_widget_queue_draw(int widget);
-	native static final protected void gtk_widget_queue_draw_area(int widget, int x, int y, int width, int height);
-	native static final protected void gtk_widget_queue_resize(int widget);
-	native static final protected void gtk_widget_size_request(int widget, int requisition);
-	native static final protected void gtk_widget_size_allocate(int widget, int allocation);
-	native static final protected void gtk_widget_get_child_requisition(int widget, int[] requisition);
-	native static final protected void gtk_widget_add_accelerator(int widget, String accelSignal, int accelGroup, int accelKey, int accelMods, int accelFlags);
-	native static final protected boolean gtk_widget_remove_accelerator(int widget, int accelGroup, int accelKey, int accelMods);
-	native static final protected void gtk_widget_set_accel_path(int widget, String accelPath, int accelGroup);
-	native static final protected int gtk_widget_list_accel_closures(int widget);
-	native static final protected boolean gtk_widget_can_activate_accel(int widget, int signalId);
-	native static final protected boolean gtk_widget_mnemonic_activate(int widget, boolean groupCycling);
-	native static final protected boolean gtk_widget_event(int widget, int event);
-	native static final protected int gtk_widget_send_expose(int widget, int event);
-	native static final protected boolean gtk_widget_activate(int widget);
-	native static final protected void gtk_widget_set_scroll_adjustments(int widget, int hadj, int vadj);
-	native static final protected void gtk_widget_reparent(int widget, int newParent);
-	native static final protected boolean gtk_widget_intersect(int widget, int area, int[] intersection);
-	native static final protected int gtk_widget_region_intersect(int widget, int region);
-	native static final protected void gtk_widget_freeze_child_notify(int widget);
-	native static final protected void gtk_widget_child_notify(int widget, String childProperty);
-	native static final protected void gtk_widget_thaw_child_notify(int widget);
-	native static final protected boolean gtk_widget_is_focus(int widget);
-	native static final protected void gtk_widget_grab_focus(int widget);
-	native static final protected void gtk_widget_grab_default(int widget);
-	native static final protected void gtk_widget_set_name(int widget, String name);
-	native static final protected String gtk_widget_get_name(int widget);
-	native static final protected void gtk_widget_set_state(int widget, int state);
-	native static final protected void gtk_widget_set_sensitive(int widget, boolean sensitive);
-	native static final protected void gtk_widget_set_app_paintable(int widget, boolean appPaintable);
-	native static final protected void gtk_widget_set_double_buffered(int widget, boolean doubleBuffered);
-	native static final protected void gtk_widget_set_redraw_on_allocate(int widget, boolean redrawOnAllocate);
-	native static final protected void gtk_widget_set_parent(int widget, int parent);
-	native static final protected void gtk_widget_set_parent_window(int widget, int parent);
-	native static final protected void gtk_widget_set_child_visible(int widget, boolean isVisible);
-	native static final protected boolean gtk_widget_get_child_visible(int widget);
-	native static final protected int gtk_widget_get_parent(int widget);
-	native static final protected int gtk_widget_get_parent_window(int widget);
-	native static final protected int gtk_widget_get_window(int widget);
-	native static final protected boolean gtk_widget_child_focus(int widget, int direction);
-	native static final protected void gtk_widget_set_size_request(int widget, int width, int height);
-	native static final protected void gtk_widget_get_size_request(int widget, int[] width, int[] height);
-	native static final protected void gtk_widget_set_events(int widget, int events);
-	native static final protected void gtk_widget_add_events(int widget, int events);
-	native static final protected void gtk_widget_set_extension_events(int widget, int mode);
-	native static final protected int gtk_widget_get_extension_events(int widget);
-	native static final protected int gtk_widget_get_toplevel(int widget);
-	native static final protected int gtk_widget_get_ancestor(int widget, int widgetType);
-	native static final protected int gtk_widget_get_colormap(int widget);
-	native static final protected int gtk_widget_get_visual(int widget);
-	native static final protected int gtk_widget_get_screen(int widget);
-	native static final protected boolean gtk_widget_has_screen(int widget); 
-	native static final protected int gtk_widget_get_display(int widget);
-	native static final protected int gtk_widget_get_root_window(int widget);
-	native static final protected boolean gtk_widget_get_sensitive(int widget);
-	native static final protected int gtk_widget_get_settings(int widget);
-	native static final protected int gtk_widget_get_accessible(int widget);
-	native static final protected void gtk_widget_set_colormap(int widget, int cmap);
-	native static final protected int gtk_widget_get_events(int widget);
-	native static final protected void gtk_widget_get_pointer(int widget, int[] x, int[] y);
-	native static final protected boolean gtk_widget_is_ancestor(int widget, int ancestor);
-	native static final protected boolean gtk_widget_translate_coordinates(int srcWidget, int destWidget, int srcX, int srcY, int[] destX, int[] destY);
-	native static final protected boolean gtk_widget_hide_on_delete(int widget);
-	native static final protected void gtk_widget_set_style(int widget, int style);
-	native static final protected void gtk_widget_ensure_style(int widget);
-	native static final protected int gtk_widget_get_style(int widget);
-	native static final protected void gtk_widget_modify_style(int widget, int style);
-	native static final protected int gtk_widget_get_modifier_style(int widget);
-	native static final protected void gtk_widget_modify_fg(int widget, int state, int color);
-	native static final protected void gtk_widget_modify_bg(int widget, int state, int color);
-	native static final protected void gtk_widget_modify_text(int widget, int state, int color);
-	native static final protected void gtk_widget_modify_base(int widget, int state, int color);
-	native static final protected void gtk_widget_modify_font(int widget, int fontDesc);
-	native static final protected int gtk_widget_create_pango_context(int widget);
-	native static final protected int gtk_widget_get_pango_context(int widget);
-	native static final protected int gtk_widget_create_pango_layout(int widget, String text);
-	native static final protected int gtk_widget_render_icon(int widget, String stockID, int size, String detail);
-	native static final protected void gtk_widget_set_composite_name(int widget, String name);
-	native static final protected String gtk_widget_get_composite_name(int widget);
-	native static final protected void gtk_widget_reset_rc_styles(int widget);
-	native static final protected void gtk_widget_push_colormap(int cmap);
+	native static final protected Handle gtk_widget_ref(Handle widget);
+	native static final protected void gtk_widget_unref(Handle widget);
+	native static final protected void gtk_widget_destroy(Handle widget);
+	native static final protected void gtk_widget_destroyed(Handle widget, Handle[] widgetPointer);
+	native static final protected void gtk_widget_unparent(Handle widget);
+	native static final protected void gtk_widget_show(Handle widget);
+	native static final protected void gtk_widget_show_now(Handle widget);
+	native static final protected void gtk_widget_hide(Handle widget);
+	native static final protected void gtk_widget_show_all(Handle widget);
+	native static final protected void gtk_widget_hide_all(Handle widget);
+	native static final protected void gtk_widget_map(Handle widget);
+	native static final protected void gtk_widget_unmap(Handle widget);
+	native static final protected void gtk_widget_realize(Handle widget);
+	native static final protected void gtk_widget_unrealize(Handle widget);
+	native static final protected void gtk_widget_queue_draw(Handle widget);
+	native static final protected void gtk_widget_queue_draw_area(Handle widget, int x, int y, int width, int height);
+	native static final protected void gtk_widget_queue_resize(Handle widget);
+	native static final protected void gtk_widget_size_request(Handle widget, Handle requisition);
+	native static final protected void gtk_widget_get_child_requisition(Handle widget, Handle[] requisition);
+	native static final protected void gtk_widget_size_allocate(Handle widget, Handle allocation);
+	native static final protected void gtk_widget_add_accelerator(Handle widget, String accelSignal, Handle accelGroup, int accelKey, int accelMods, int accelFlags);
+	native static final protected boolean gtk_widget_remove_accelerator(Handle widget, Handle accelGroup, int accelKey, int accelMods);
+	native static final protected void gtk_widget_set_accel_path(Handle widget, String accelPath, Handle accelGroup);
+	native static final protected int gtk_widget_list_accel_closures(Handle widget);
+	native static final protected boolean gtk_widget_can_activate_accel(Handle widget, int signalId);
+	native static final protected boolean gtk_widget_mnemonic_activate(Handle widget, boolean groupCycling);
+	native static final protected boolean gtk_widget_event(Handle widget, Handle event);
+	native static final protected int gtk_widget_send_expose(Handle widget, Handle event);
+	native static final protected boolean gtk_widget_activate(Handle widget);
+	native static final protected void gtk_widget_set_scroll_adjustments(Handle widget, Handle hadj, Handle vadj);
+	native static final protected void gtk_widget_reparent(Handle widget, Handle newParent);
+	native static final protected boolean gtk_widget_intersect(Handle widget, Handle area, Handle intersection);
+	native static final protected int gtk_widget_region_intersect(Handle widget, Handle region);
+	native static final protected void gtk_widget_freeze_child_notify(Handle widget);
+	native static final protected void gtk_widget_child_notify(Handle widget, String childProperty);
+	native static final protected void gtk_widget_thaw_child_notify(Handle widget);
+	native static final protected boolean gtk_widget_is_focus(Handle widget);
+	native static final protected void gtk_widget_grab_focus(Handle widget);
+	native static final protected void gtk_widget_grab_default(Handle widget);
+	native static final protected void gtk_widget_set_name(Handle widget, String name);
+	native static final protected String gtk_widget_get_name(Handle widget);
+	native static final protected void gtk_widget_set_state(Handle widget, int state);
+	native static final protected void gtk_widget_set_sensitive(Handle widget, boolean sensitive);
+	native static final protected void gtk_widget_set_app_paintable(Handle widget, boolean appPaintable);
+	native static final protected void gtk_widget_set_double_buffered(Handle widget, boolean doubleBuffered);
+	native static final protected void gtk_widget_set_redraw_on_allocate(Handle widget, boolean redrawOnAllocate);
+	native static final protected void gtk_widget_set_parent(Handle widget, Handle parent);
+	native static final protected void gtk_widget_set_parent_window(Handle widget, Handle parent);
+	native static final protected void gtk_widget_set_child_visible(Handle widget, boolean isVisible);
+	native static final protected boolean gtk_widget_get_child_visible(Handle widget);
+	native static final protected Handle gtk_widget_get_parent(Handle widget);
+	native static final protected Handle gtk_widget_get_parent_window(Handle widget);
+	native static final protected Handle gtk_widget_get_window(Handle widget);
+	native static final protected boolean gtk_widget_child_focus(Handle widget, int direction);
+	native static final protected void gtk_widget_set_size_request(Handle widget, int width, int height);
+	native static final protected void gtk_widget_get_size_request(Handle widget, int[] width, int[] height);
+	native static final protected void gtk_widget_set_events(Handle widget, int events);
+	native static final protected void gtk_widget_add_events(Handle widget, int events);
+	native static final protected void gtk_widget_set_extension_events(Handle widget, int mode);
+	native static final protected int gtk_widget_get_extension_events(Handle widget);
+	native static final protected Handle gtk_widget_get_toplevel(Handle widget);
+	native static final protected Handle gtk_widget_get_ancestor(Handle widget, int widgetType);
+	native static final protected Handle gtk_widget_get_colormap(Handle widget);
+	native static final protected Handle gtk_widget_get_visual(Handle widget);
+	native static final protected Handle gtk_widget_get_screen(Handle widget);
+	native static final protected boolean gtk_widget_has_screen(Handle widget); 
+	native static final protected Handle gtk_widget_get_display(Handle widget);
+	native static final protected Handle gtk_widget_get_root_window(Handle widget);
+	native static final protected boolean gtk_widget_get_sensitive(Handle widget);
+	native static final protected Handle gtk_widget_get_settings(Handle widget);
+	native static final protected Handle gtk_widget_get_accessible(Handle widget);
+	native static final protected void gtk_widget_set_colormap(Handle widget, Handle cmap);
+	native static final protected int gtk_widget_get_events(Handle widget);
+	native static final protected void gtk_widget_get_pointer(Handle widget, int[] x, int[] y);
+	native static final protected boolean gtk_widget_is_ancestor(Handle widget, Handle ancestor);
+	native static final protected boolean gtk_widget_translate_coordinates(Handle srcWidget, Handle destWidget, int srcX, int srcY, int[] destX, int[] destY);
+	native static final protected boolean gtk_widget_hide_on_delete(Handle widget);
+	native static final protected void gtk_widget_set_style(Handle widget, Handle style);
+	native static final protected void gtk_widget_ensure_style(Handle widget);
+	native static final protected Handle gtk_widget_get_style(Handle widget);
+	native static final protected void gtk_widget_modify_style(Handle widget, Handle style);
+	native static final protected Handle gtk_widget_get_modifier_style(Handle widget);
+	native static final protected void gtk_widget_modify_fg(Handle widget, int state, Handle color);
+	native static final protected void gtk_widget_modify_bg(Handle widget, int state, Handle color);
+	native static final protected void gtk_widget_modify_text(Handle widget, int state, Handle color);
+	native static final protected void gtk_widget_modify_base(Handle widget, int state, Handle color);
+	native static final protected void gtk_widget_modify_font(Handle widget, Handle fontDesc);
+	native static final protected Handle gtk_widget_create_pango_context(Handle widget);
+	native static final protected Handle gtk_widget_get_pango_context(Handle widget);
+	native static final protected Handle gtk_widget_create_pango_layout(Handle widget, String text);
+	native static final protected Handle gtk_widget_render_icon(Handle widget, String stockID, int size, String detail);
+	native static final protected void gtk_widget_set_composite_name(Handle widget, String name);
+	native static final protected String gtk_widget_get_composite_name(Handle widget);
+	native static final protected void gtk_widget_reset_rc_styles(Handle widget);
+	native static final protected void gtk_widget_push_colormap(Handle cmap);
 	native static final protected void gtk_widget_pop_colormap();
 	native static final protected void gtk_widget_push_composite_child();
 	native static final protected void gtk_widget_pop_composite_child();
-	native static final protected void gtk_widget_style_get_property(int widget, String propertyName, int value);
-	native static final protected void gtk_widget_set_default_colormap(int cmap);
-	native static final protected int gtk_widget_get_default_colormap();
-	native static final protected int gtk_widget_get_default_visual();
-	native static final protected int gtk_widget_get_default_style();
-	native static final protected void gtk_widget_set_direction(int widget, int dir);
-	native static final protected int gtk_widget_get_direction(int widget);
+	native static final protected void gtk_widget_style_get_property(Handle widget, String propertyName, Handle value);
+	native static final protected void gtk_widget_set_default_colormap(Handle cmap);
+	native static final protected Handle gtk_widget_get_default_colormap();
+	native static final protected Handle gtk_widget_get_default_visual();
+	native static final protected Handle gtk_widget_get_default_style();
+	native static final protected void gtk_widget_set_direction(Handle widget, int dir);
+	native static final protected int gtk_widget_get_direction(Handle widget);
 	native static final protected void gtk_widget_set_default_direction(int dir);
 	native static final protected int gtk_widget_get_default_direction();
-	native static final protected void gtk_widget_shape_combine_mask(int widget, int shapeMask, int offsetX, int offsetY); 
-	native static final protected void gtk_widget_path(int widget, int[] pathLength, String[] path, String[] pathReversed); 
-	native static final protected void gtk_widget_class_path(int widget, int[] pathLength, String[] path, String[] pathReversed);
-	native static final protected int[] gtk_widget_list_mnemonic_labels(int widget);
-	native static final protected void gtk_widget_add_mnemonic_label(int widget, int label);
-	native static final protected void gtk_widget_remove_mnemonic_label(int widget, int label);
+	native static final protected void gtk_widget_shape_combine_mask(Handle widget, Handle shapeMask, int offsetX, int offsetY); 
+	native static final protected void gtk_widget_path(Handle widget, int[] pathLength, String[] path, String[] pathReversed); 
+	native static final protected void gtk_widget_class_path(Handle widget, int[] pathLength, String[] path, String[] pathReversed);
+	native static final protected Handle[] gtk_widget_list_mnemonic_labels(Handle widget);
+	native static final protected void gtk_widget_add_mnemonic_label(Handle widget, Handle label);
+	native static final protected void gtk_widget_remove_mnemonic_label(Handle widget, Handle label);
 	// Drag and Drop support
-	native static final protected void gtk_drag_dest_set(int widget, int flags, int[] targets, int actions);
-	native static final protected void gtk_drag_dest_unset(int widget);
-	native static final protected void gtk_drag_get_data(int widget, int context, int target);
-	native static final protected void gtk_drag_finish(int context, boolean success, boolean delete);
-	native static final protected void gtk_drag_highlight(int widget);
-	native static final protected void gtk_drag_unhighlight(int widget);
-	native static final protected void gtk_drag_source_set(int widget, int start_button_mask, int[] targets, int actions);
-	native static final protected void gtk_drag_source_unset(int widget);
-	native static final protected void gtk_drag_source_set_icon(int widget, int colormap, int pixmap, int mask);
-	native static final protected void gtk_drag_source_set_icon_pixbuf(int widget, int pixbuf);
-	native static final protected void gtk_drag_source_set_icon_stock(int widget, String stockId);
-	/****************************************
-	 * END OF JNI CODE
-	 ****************************************/
+	native static final protected void gtk_drag_source_set(Handle widget, int start_button_mask, Handle[] targets, int actions);
+	native static final protected void gtk_drag_source_unset(Handle widget);
+	native static final protected void gtk_drag_source_set_icon(Handle widget, Handle colormap, Handle pixmap, Handle mask);
+	native static final protected void gtk_drag_source_set_icon_pixbuf(Handle widget, Handle pixbuf);
+	native static final protected void gtk_drag_source_set_icon_stock(Handle widget, String stockId);
+	native static final protected void gtk_drag_get_data(Handle widget, Handle context, int target);
+	native static final protected void gtk_drag_finish(Handle context, boolean success, boolean delete);
+	native static final protected void gtk_drag_highlight(Handle widget);
+	native static final protected void gtk_drag_unhighlight(Handle widget);
+	native static final protected void gtk_drag_dest_set(Handle widget, int flags, Handle[] targets, int actions);
+	native static final protected void gtk_drag_dest_unset(Handle widget);
 }
