@@ -311,13 +311,13 @@ jg_arg_signature(GString *str, GType argtype)
     as = "java/lang/String";
     break;
   case G_TYPE_BOXED:
-    ac = 'I';
+    as = "org/gnu/glib/Handle";
     break;
   case G_TYPE_POINTER:
     ac = 'I';
     break;
   case G_TYPE_OBJECT:
-    ac = 'I';
+    as = "org/gnu/glib/Handle";
     break;
   default:
     ac = 'I';
@@ -518,8 +518,6 @@ static jint create_callback(JNIEnv *env,
 
 //  go = (GObject*)getPointerFromHandle(env, obj);
   go = (GObject*)getPointerFromJavaGObject(env, obj);
-  if (go == NULL)
-  	printf("go is NULL\n");
   signame = (*env)->GetStringUTFChars(env, sigstring, NULL);
   sigfunc = (*env)->GetStringUTFChars(env, func, NULL);
   
@@ -709,50 +707,45 @@ JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_getData
 	}
 }
 
-
-static void
-jg_gobject_destroy(gpointer data)
+static void jg_gobject_destroy(gpointer data)
 {
 	JGRef *ref = (JGRef*) data;
 	(* ref->env)->DeleteGlobalRef( ref->env, ref->object);
 	g_free( ref );
 }
+
 /*
  * Class:     org_gnu_glib_GObject
  * Method:    setData
  * Signature: (ILjava/lang/String;Ljava/lang/Object;)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_setData
-  (JNIEnv *env, jobject obj, jobject handle, jstring key, jobject data)
+(JNIEnv *env, jclass cls, jobject handle, jstring key, jobject data)
 {
-        GObject *handle_g = (GObject *)getPointerFromHandle(env, handle);
-	gchar *k = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	JGRef *ref = g_new( JGRef, 1 );
-	ref->env = env;
-	ref->object = (* env)->NewGlobalRef(env, data);
-	g_object_set_data_full(handle_g,
-				k, 
-				ref,
-				jg_gobject_destroy
-				) ;
-	(*env)->ReleaseStringUTFChars(env, key, k);
+    GObject *handle_g = (GObject *)getPointerFromHandle(env, handle);
+    gchar *k = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
+
+    JGRef *ref = g_new( JGRef, 1 );
+    ref->env = env;
+    ref->object = (* env)->NewGlobalRef(env, data);
+
+    g_object_set_data_full(handle_g, k, ref, jg_gobject_destroy ) ;
+    (*env)->ReleaseStringUTFChars(env, key, k);
 }
 
 void logFunc(const gchar* logDomain, GLogLevelFlags logLevel, const gchar* message, gpointer userData)
 {
-	jclass gobject;
-	jmethodID stackHandler;
-	JNIEnv *env = (JNIEnv*)userData;
-	jstring msg;
-	jvalue *jargs;
+    jclass gobject;
+    jmethodID stackHandler;
+    JNIEnv *env = (JNIEnv*)userData;
+    jstring msg;
 	
-	msg = (*env)->NewStringUTF(env, message);
-	jargs = alloca(sizeof(jvalue));
-    jargs[0].l = msg;
+    (*env)->ExceptionClear(env);
+    msg = (*env)->NewStringUTF(env, message);
 	
-	gobject = (*env)->FindClass(env, "Lorg/gnu/glib/GObject;");
-	stackHandler = (*env)->GetMethodID(env, gobject, "printStackTrace", "(Ljava/lang/String;)V");
-    (*env)->CallStaticVoidMethodA(env, gobject, stackHandler, jargs);
+    gobject = (*env)->FindClass(env, "org/gnu/glib/GObject");
+    stackHandler = (*env)->GetStaticMethodID(env, gobject, "printStackTrace", "(Ljava/lang/String;)V");
+    (*env)->CallStaticVoidMethod(env, gobject, stackHandler, msg);
 	
 }
 
@@ -760,11 +753,22 @@ void logFunc(const gchar* logDomain, GLogLevelFlags logLevel, const gchar* messa
  * Class:     org_gnu_glib_GObject
  * Method:    registerLogHandler
  * Signature: ()V
- */ JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_registerLogHandler
-  (JNIEnv *env, jclass cls)
+ */ 
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_registerLogHandler
+(JNIEnv *env, jclass cls)
 {
 	g_log_set_handler(NULL, G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL |
 			G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, logFunc, (gpointer)env);
+	g_log_set_handler ("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+                     | G_LOG_FLAG_RECURSION, logFunc, (gpointer) env);
+	g_log_set_handler ("Gnome", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+                     | G_LOG_FLAG_RECURSION, logFunc, (gpointer) env);
+	g_log_set_handler ("Gtk", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+                     | G_LOG_FLAG_RECURSION, logFunc, (gpointer) env);
+	g_log_set_handler ("Gconf", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+                     | G_LOG_FLAG_RECURSION, logFunc, (gpointer) env);
+	g_log_set_handler ("Glade", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+                     | G_LOG_FLAG_RECURSION, logFunc, (gpointer) env);
 }
 
 /*
@@ -778,6 +782,137 @@ JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_getNullHandle
     return getHandleFromPointer(env, NULL);
 }
 
+JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_g_1object_1new
+(JNIEnv *env, jclass cls, jint object_type)
+{
+    GType object_type_g = (GType)object_type;
+    return getHandleFromPointer(env, g_object_new(object_type_g, NULL, NULL));
+}
+
+JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_g_1object_1class_1find_1property
+(JNIEnv *env, jclass cls, jobject oclass, jstring property_name)
+{
+    GObjectClass * oclass_g = 
+        G_OBJECT_GET_CLASS((GObject *)getPointerFromHandle(env, oclass));
+    const gchar * property_name_g = 
+        (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
+
+    jobject ret_g = getHandleFromPointer(env, g_object_class_find_property(oclass_g, property_name_g));
+
+    (*env)->ReleaseStringUTFChars(env, property_name, property_name_g);
+    return ret_g;
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_g_1object_1set_1property
+(JNIEnv *env, jclass cls, jobject object, jstring property_name, jobject value)
+{
+    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+    const gchar * property_name_g = (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
+    const GValue * value_g = (const GValue *)getPointerFromHandle(env, value);
+    g_object_set_property(object_g, property_name_g, value_g);
+    (*env)->ReleaseStringUTFChars(env, property_name, property_name_g);
+}
+
+JNIEXPORT jobject JNICALL Java_org_gnu_glib_GObject_g_1object_1get_1property
+(JNIEnv *env, jclass cls, jobject object, jstring property_name)
+{
+    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+    const gchar * property_name_g = 
+        (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
+
+    // Initialize the GValue.
+    GParamSpec *pspec = 
+        g_object_class_find_property(G_OBJECT_GET_CLASS(object_g), 
+                                     property_name_g);
+    if ( !pspec ) {
+        return NULL;
+    }
+    GValue * value_g = g_malloc( sizeof( GValue ) );
+    value_g->g_type = 0;
+    value_g = g_value_init(value_g, pspec->value_type);
+
+    // Get the property.
+    g_object_get_property(object_g, property_name_g, value_g);
+    (*env)->ReleaseStringUTFChars(env, property_name, property_name_g);
+    return getHandleFromPointer(env, value_g);
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_g_1object_1freeze_1notify
+(JNIEnv *env, jclass cls, jobject object)
+{
+    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+    g_object_freeze_notify(object_g);
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_g_1object_1notify
+(JNIEnv *env, jclass cls, jobject object, jstring property_name)
+{
+    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+    const gchar * property_name_g = (const gchar *)(*env)->GetStringUTFChars(env, property_name, 0);
+    g_object_notify(object_g, property_name_g);
+    (*env)->ReleaseStringUTFChars(env, property_name, property_name_g);
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_g_1object_1thaw_1notify
+(JNIEnv *env, jclass cls, jobject object)
+{
+    GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+    g_object_thaw_notify(object_g);
+}
+
+static JGFuncCallbackRef *propertyNotifRef = NULL;
+
+static void propertyNotificationCB(GObject *gobject, GParamSpec *arg1,
+                                   gpointer data)
+{
+    JGFuncCallbackRef *ref = (JGFuncCallbackRef*) data;
+    (* ref->env)->CallVoidMethod(ref->env, 
+                                 ref->obj, 
+                                 ref->methodID,
+                                 getHandleFromPointer(ref->env, gobject),
+                                 (*ref->env)->NewStringUTF(ref->env, 
+                                                           arg1->name));
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_connectNotifySignal
+(JNIEnv *env, jclass cls, jobject objhandle, jobject object, jstring callback)
+{
+    propertyNotifRef = g_new( JGFuncCallbackRef, 1 );
+    propertyNotifRef->env = env;
+    propertyNotifRef->obj = (* env)->NewGlobalRef(env, object);
+
+    const char *funcname = (*env)->GetStringUTFChars(env, callback, NULL);
+    const char *sig = "(Lorg/gnu/glib/Handle;Ljava/lang/String;)V";
+    // Get method id for the callback method name.
+    propertyNotifRef->methodID = 
+        (*env)->GetMethodID(env, 
+                            (*env)->GetObjectClass(env, propertyNotifRef->obj),
+                            funcname, sig );
+    if ( propertyNotifRef->methodID == NULL ) {
+        g_warning( "Can't find %s%s.\n", funcname, sig );
+        (*env)->ReleaseStringUTFChars(env, callback, funcname);
+        g_free( propertyNotifRef );
+        propertyNotifRef = NULL;
+        return;
+    }
+    (*env)->ReleaseStringUTFChars(env, callback, funcname);
+
+    GObject * object_g = (GObject *)getPointerFromHandle(env, objhandle);
+    g_object_connect(object_g, "signal::notify", 
+                     propertyNotificationCB, propertyNotifRef, NULL);
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_glib_GObject_disconnectNotifySignal
+(JNIEnv *env, jclass cls, jobject object)
+{
+    if ( propertyNotifRef ) {
+        GObject * object_g = (GObject *)getPointerFromHandle(env, object);
+        g_object_disconnect(object_g, "any_signal::notify",
+                            propertyNotificationCB, propertyNotifRef, NULL);
+        g_free( propertyNotifRef );
+    }
+}
+
 #ifdef __cplusplus
 }
 
