@@ -178,6 +178,15 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_SelectionData_gtk_0selection_1data_1
     return (jboolean)gtk_selection_data_targets_include_text(data_g);
 }
 
+JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_SelectionData_gtk_1selection_1data_1targets_1include_1image
+(JNIEnv *env, jclass cls, jobject selection_data, jboolean writable)
+{
+    GtkSelectionData * selection_data_g = 
+        (GtkSelectionData *)getPointerFromHandle(env, selection_data);
+    gboolean writable_g = (gboolean)writable;
+    return (jboolean)gtk_selection_data_targets_include_image(selection_data_g, writable_g);
+}
+
 /*
  * Class:     org_gnu_gtk_SelectionData
  * Method:    gtk_selection_data_set_pixbuf
@@ -227,7 +236,6 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_SelectionData_gtk_1selection_1da
    return getJavaStringArray(env, str);
 }
 
-
 #ifdef __cplusplus
 }
 
